;;; Lepton EDA library - Scheme API
;;; Copyright (C) 2022 Lepton EDA Contributors
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


(define-module (lepton toplevel)
  #:use-module (ice-9 format)
  #:use-module (system foreign)

  #:use-module (lepton ffi)
  #:use-module (lepton toplevel foreign)

  #:export (%lepton-toplevel
            toplevel?
            current-toplevel
            set-current-toplevel!
            make-toplevel
            with-toplevel))

;;; Initialize %lepton-toplevel with a new fluid variable for
;;; Scheme and C code.
;;; This fluid is used for setting or getting the <toplevel>
;;; instance associated with the current dynamic context.
(define %lepton-toplevel
  (let ((toplevel-fluid (make-fluid)))
    (lepton_init_toplevel_fluid (scm->pointer toplevel-fluid))
    toplevel-fluid))


(define (toplevel? toplevel)
  "Returns #t if TOPLEVEL is a <toplevel> instance, otherwise
returns #f."
  (is-toplevel? toplevel))



(define (make-toplevel)
  "Makes and returns a new <toplevel> type instance."
  (pointer->toplevel (lepton_toplevel_new)))

(define (current-toplevel)
  "Returns the <toplevel> instance associated with the current
dynamic context."
  (and=> (fluid-ref %lepton-toplevel) pointer->toplevel))


(define (set-current-toplevel! toplevel)
  "Unconditionally set current toplevel to TOPLEVEL which must be
a <toplevel> instance."
  (fluid-set! %lepton-toplevel (toplevel->pointer toplevel)))


(define (with-toplevel toplevel thunk)
  "Call THUNK in the dynamic state of TOPLEVEL which must be a
<toplevel> instance."
  (with-fluid* %lepton-toplevel
    (toplevel->pointer toplevel) thunk))
